import ipaddress

from infoblox_client import connector, objects

from gso.settings import load_oss_params


def _setup_connection() -> connector.Connector:
    oss = load_oss_params()
    options = {
        "host": oss.IPAM.INFOBLOX.host,
        "username": oss.IPAM.INFOBLOX.username,
        "password": oss.IPAM.INFOBLOX.password,
        "wapi_version": oss.IPAM.INFOBLOX.wapi_version[1:],  # remove the 'v' in front of the version number
        "ssl_verify": True if oss.IPAM.INFOBLOX.scheme == "https" else False,
    }
    return connector.Connector(options)


def allocate_network(ip_network: ipaddress.IPv4Network | ipaddress.IPv6Network) -> objects.Network:
    conn = _setup_connection()
    return objects.Network.create(conn, str(ip_network))


def delete_network(ip_network: ipaddress.IPv4Network | ipaddress.IPv6Network) -> None:
    conn = _setup_connection()
    network = objects.Network.search(conn, cidr=str(ip_network))
    network.delete()


def allocate_host(hostname: str, net_cidr: ipaddress.IPv4Network | ipaddress.IPv6Network) -> objects.HostRecord:
    conn = _setup_connection()
    address = objects.IPAllocation.next_available_ip_from_cidr("default", net_cidr)
    ip_object = objects.IP.create(address, "00:00:00:00:00:00", configure_for_dhcp=False)
    return objects.HostRecord.create(conn, name=hostname, ip=ip_object)


def delete_host_by_ip(ip_addr: ipaddress.IPv4Address | ipaddress.IPv6Address) -> None:
    conn = _setup_connection()
    host = objects.HostRecord.search(conn, ipv4addr=ip_addr)
    host.delete()


def delete_host_by_fqdn(fqdn: str) -> None:
    conn = _setup_connection()
    host = objects.HostRecord.search(conn, name=fqdn)
    host.delete()
