# mypy: ignore-errors
from enum import Enum, auto
from typing import List

from gso import settings

# TODO
# - fill in the implementations
# - consider the extra API methods
# - decided what to do with various error conditions (currently assertions)


class InterfaceAllocationState(Enum):
    AVAILABLE = auto()
    RESERVED = auto()
    ALLOCATED = auto()


def _dummy_router_interfaces():
    return {
        "lags": [],
        "physical": [{"name": f"ifc-{x}", "state": InterfaceAllocationState.AVAILABLE} for x in range(250)],
    }


_DUMMY_INVENTORY = {
    "fqdn-a": _dummy_router_interfaces(),
    "fqdn-b": _dummy_router_interfaces(),
    "fqdn-c": _dummy_router_interfaces(),
    "fqdn-d": _dummy_router_interfaces(),
}


def import_new_router(new_router_fqdn: str, subscription_id: str, oss_params=settings.OSSParams):
    # TODO: this is a dummy implementation

    # TODO: specifiy if this should be an error (and if now, what it means)
    assert new_router_fqdn not in _DUMMY_INVENTORY
    _DUMMY_INVENTORY[new_router_fqdn] = _dummy_router_interfaces()


def next_lag(router_fqdn: str, subscription_id: str, oss_params=settings.OSSParams) -> str:
    # TODO: this is a dummy implementation

    assert router_fqdn in _DUMMY_INVENTORY

    lag_idx = 0
    while True:
        lag_name = f"ae-{lag_idx}"
        if lag_name not in _DUMMY_INVENTORY[router_fqdn]["lags"]:
            _DUMMY_INVENTORY[router_fqdn]["lags"].append(lag_name)
            return lag_name
        lag_idx += 1


def available_physical_interfaces(router_fqdn: str, oss_params=settings.OSSParams) -> List[str]:
    # TODO: this is a dummy implementation

    assert router_fqdn in _DUMMY_INVENTORY

    return [
        ifc["name"]
        for ifc in _DUMMY_INVENTORY[router_fqdn]["physical"]
        if ifc["state"] == InterfaceAllocationState.AVAILABLE
    ]


def _find_physical(router_fqdn: str, interface_name: str) -> dict:
    assert router_fqdn in _DUMMY_INVENTORY
    for ifc in _DUMMY_INVENTORY[router_fqdn]["physical"]:
        if ifc["name"] == interface_name:
            return ifc
    raise AssertionError(f"interface {interface_name} not found on {router_fqdn}")


def reserve_physical_interface(
    router_fqdn: str, interface_name: str, subscription_id: str, oss_params=settings.OSSParams
):
    # TODO: this is a dummy implementation

    ifc = _find_physical(router_fqdn, interface_name)
    assert (
        ifc["state"] == InterfaceAllocationState.AVAILABLE
    ), f"interface {router_fqdn}:{interface_name} is not available"
    ifc["state"] = InterfaceAllocationState.RESERVED


def allocate_physical_interface(router_fqdn: str, interface_name: str, oss_params=settings.OSSParams):
    # TODO: this is a dummy implementation

    ifc = _find_physical(router_fqdn, interface_name)
    # TODO: is there a use case for moving
    #  directly from AVAILABLE to ALLOCATED?
    assert (
        ifc["state"] == InterfaceAllocationState.RESERVED
    ), f"interface {router_fqdn}:{interface_name} is not reserved"
    ifc["state"] = InterfaceAllocationState.RESERVED


def free_physical_interface(router_fqdn: str, interface_name: str, oss_params=settings.OSSParams):
    # TODO: this is a dummy implementation

    ifc = _find_physical(router_fqdn, interface_name)
    # TODO: is this truly an error that should be handled?
    # or is it ok to ignore this?
    assert (
        ifc["state"] != InterfaceAllocationState.AVAILABLE
    ), f"interface {router_fqdn}:{interface_name} is already available"
    ifc["state"] = InterfaceAllocationState.AVAILABLE


def free_lag(router_fqdn: str, lag_name: str, oss_params=settings.OSSParams):
    # TODO: is this a use case that should be handled?
    # e.g. who keeps track of the bundled physical interfaces?
    pass


def remove_router(router_fqdn: str, oss_params=settings.OSSParams):
    # TODO: is this a use case that should be handled?
    pass


def all_lags(router_fqdn: str, oss_params=settings.OSSParams) -> List[str]:
    # TODO: is this a use case that should be handled?
    assert router_fqdn in _DUMMY_INVENTORY
    return _DUMMY_INVENTORY[router_fqdn]["lags"]
